import es
import gamethread
import random
import popuplib

####--Addon Info(Do not touch)--####
info = es.AddonInfo()
info.name        = "Dominating for CS:Source"
info.version     = "2.2.0"
info.author      = "TheBash"
info.url         = "http://addons.eventscripts.com/addons/view/dominating"
info.basename    = "dominating"
info.description = "Gives CS:Source the 'dominating' functionality from TF2, with a few extra twists!"

####--Dominating Level Control--####
dom_ranges = {                                  #Set this to a dictionary of levels (as many levels as you want)
    3: "dominated",
    6: "obliterated",
    10: "totally pwned",
    20: "made useless"
}

####--Dominating Reset Control--####
dom_reset_on_map_change = True                  #If True, will reset all dominations on a map change
dom_reset_on_team_change = True                 #If True, will reset dominations on a player if they change teams
dom_reset_on_disconnect = True                  #If True, will reset dominations on a player if they disconnect

####--Dominating Popup Control--####
dom_popup_top_value = 5                         #Sets the number of entries to show in the best/most dominations popups
dom_popup_help_cmd = "!domhelp"                 #Sets the say command to show the help popup
dom_popup_list_ranges_cmd = "!domlevels"        #Sets the say command to show the domination levels popup
dom_popup_current_cmd = "!currentdom"           #Sets the say command to show current dominations popup
dom_popup_most_cmd = "!mostdom"                 #Sets the say command to show most dominations popup
dom_popup_best_cmd = "!bestdom"                 #Sets the say command to show best dominations popup
dom_popup_mydoms_cmd = "!mydom"                 #Sets the say command to show personal dominatons popup (per player basis)

####--Dominating Sound Control--####
dom_use_sounds = False                          #If True, will use sounds
dom_nemesis_sound = "tf2sounds/tf2_nemesis.mp3" #Set this to the sound/music to use when a nemesis is made
dom_revenge_sound = "tf2sounds/tf2_revenge.mp3" #Set this to the sound/music to use when a revenge is made
dom_taunts = [                                  #Set this to a list of sounds to use as taunts (as many taunts as you want)
    "tf2sounds/taunts/taunts1.mp3",
    "tf2sounds/taunts/taunts2.mp3",
    "tf2sounds/taunts/taunts3.mp3",
    "tf2sounds/taunts/taunts4.mp3",
    "tf2sounds/taunts/taunts5.mp3",
    "tf2sounds/taunts/taunts6.mp3",
    "tf2sounds/taunts/taunts7.mp3",
    "tf2sounds/taunts/taunts8.mp3",
    "tf2sounds/taunts/taunts9.mp3",
    "tf2sounds/taunts/taunts10.mp3",
    "tf2sounds/taunts/taunts11.mp3",
    "tf2sounds/taunts/taunts12.mp3",
    "tf2sounds/taunts/taunts13.mp3",
    "tf2sounds/taunts/taunts14.mp3",
    "tf2sounds/taunts/taunts15.mp3",
    "tf2sounds/taunts/taunts16.mp3"
]
dom_taunt_delay = 1.6                           #Set this to the delay that the taunt will play at after the domination/nemesis event.
dom_sound_mode = 0                              #If 0, will play to all people; If 1, will play to surrounding people; If 2, will play only to players involved in the event.
dom_sound_music_volume = 0.9                    #Set the volume of the musics (0.0 - 1.0)
dom_sound_taunt_volume = 1.0                    #Set the volume of the taunts (0.0 - 1.0)
dom_sound_attenuation = 0.5                     #Sets the attenuation of the taunt sound (0.0 - 1.0) {{This variable is only used when dom_sound_mode is 1}}                  
dom_sound_mute_cmd = "!dommute"                 #Sets the say command to set sounds to mute (per player basis)
dom_sound_unmute_cmd = "!domunmute"             #Sets the say command to set sounds to unmute (per player basis)

####--Dominating Visual Control--####
dom_show_centre_locally = False                 #If True, will only show centre message to players involved in the event.

####--DO NOT EDIT BELOW THIS LINE!--####

dominations = {}
domination_counts = {}
domination_records = {}
muted = {}

first_range = -1
for dom_range in dom_ranges:
    if dom_range < first_range or first_range == -1:
        first_range = dom_range

dom_popup = None

def register_downloads():
    es.stringtable("downloadables", "sound/" + dom_nemesis_sound)
    es.stringtable("downloadables", "sound/" + dom_revenge_sound)
    for taunt in dom_taunts:
        es.stringtable("downloadables", "sound/" + taunt)

def load():
    es.set("dominating_version",info.version)
    es.makepublic("dominating_version")
    
    if not es.exists('saycommand', dom_popup_help_cmd):
        es.regsaycmd(dom_popup_help_cmd, "dominating/show_help", 'Shows help')
    
    if not es.exists('saycommand', dom_popup_mydoms_cmd):
        es.regsaycmd(dom_popup_mydoms_cmd, "dominating/show_your_dominations", 'Shows your dominations')

    if not es.exists('saycommand', dom_popup_current_cmd):
        es.regsaycmd(dom_popup_current_cmd, "dominating/show_current_dominations", 'Shows current dominations')

    if not es.exists('saycommand', dom_popup_most_cmd):
        es.regsaycmd(dom_popup_most_cmd, "dominating/show_most_dominations", 'Shows most dominations')

    if not es.exists('saycommand', dom_popup_best_cmd):
        es.regsaycmd(dom_popup_best_cmd, "dominating/show_best_dominations", 'Shows best dominations')

    if not es.exists('saycommand', dom_popup_list_ranges_cmd):
        es.regsaycmd(dom_popup_list_ranges_cmd, "dominating/show_levels", 'Shows domination levels')

    random.seed()

    if dom_use_sounds:
        if not es.exists('saycommand', dom_sound_mute_cmd):
            es.regsaycmd(dom_sound_mute_cmd, "dominating/mute_sounds", 'Mute sounds')
    
        if not es.exists('saycommand', dom_sound_unmute_cmd):
            es.regsaycmd(dom_sound_unmute_cmd, "dominating/unmute_sounds", 'Unmute sounds')
        
        register_downloads()
        
def unload():
    global dominations
    global domination_counts
    global domination_records
    dominations.clear()
    domination_counts.clear()
    domination_records.clear()
    es.unregsaycmd(dom_popup_help_cmd)
    es.unregsaycmd(dom_popup_mydoms_cmd)
    es.unregsaycmd(dom_popup_current_cmd)
    es.unregsaycmd(dom_popup_most_cmd)
    es.unregsaycmd(dom_popup_best_cmd)
    es.unregsaycmd(dom_popup_list_ranges_cmd)
    
    if dom_use_sounds:
        es.unregsaycmd(dom_sound_mute_cmd)
        es.unregsaycmd(dom_sound_unmute_cmd)

def es_map_start(event_var):
    global dominations
    global domination_counts
    global domination_records
    if dom_reset_on_map_change:
        dominations.clear()
        domination_counts.clear()
        domination_records.clear()
        
    if dom_use_sounds:
        register_downloads()
        
def player_team(event_var):
    global dominations
    global dom_reset_on_team_change
    if dom_reset_on_team_change:
        for key in dominations.keys():
            if event_var['userid'] in key.split('<'):
                del dominations[key]

def player_disconnect(event_var):
    global dominations
    global muted
    if event_var['userid'] in muted.keys():
        del muted[event_var['userid']]
    if dom_reset_on_disconnect:
        for key in dominations.keys():
            if event_var['userid'] in key.split('<'):
                del dominations[key]

def play_taunt():
    if dom_use_sounds:
        number = random.randint(0, len(dom_taunts))
        for ply in es.getUseridList():
            if not ply in muted.keys():
                es.playsound(ply, dom_taunts[number], dom_sound_taunt_volume)
                
def play_taunt_local(playera, playerb):
    if dom_use_sounds:
        number = random.randint(0, len(dom_taunts))
        if dom_sound_mode == 1:
            es.emitsound("player", playera, dom_taunts[number], dom_sound_taunt_volume, dom_sound_attenuation)
        else:
            if not playera in muted.keys():
                es.playsound(playera, dom_taunts[number], dom_sound_taunt_volume)
            if not playerb in muted.keys():
                es.playsound(playerb, dom_taunts[number], dom_sound_taunt_volume)
            
def play_sound(sound_name):
    if dom_use_sounds:
        for ply in es.getUseridList():
            if not ply in muted.keys():
                es.playsound(ply, sound_name, dom_sound_music_volume)

def play_sound_local(sound_name, playera, playerb):
    if dom_use_sounds:
        if dom_sound_mode == 1:
            es.emitsound("player", playera, sound_name, dom_sound_music_volume, dom_sound_attenuation)
        else:
            if not playera in muted.keys():
                es.playsound(playera, sound_name, dom_sound_music_volume)
            if not playerb in muted.keys():
                es.playsound(playerb, sound_name, dom_sound_music_volume)
    
            
def player_death(event_var):
    global dominations
    global domination_counts
    global domination_records

    centermess = ''
    str1 = event_var['userid'] + '<' + event_var['attacker']
    str2 = event_var['attacker'] + '<' + event_var['userid']
    if str1 in dominations.keys():
        dominations[str1] += 1
        if str2 in dominations.keys():
            if dominations[str2] >= first_range: 
                es.msg('#multi', '#green[DOM]#default ' + es.getplayername(event_var['attacker']) + ' got revenge on#green ' + es.getplayername(event_var['userid']) + '#default!')
                if dom_sound_mode > 0:
                    play_sound_local(dom_revenge_sound, event_var['attacker'], event_var['userid'])
                else:
                    play_sound(dom_revenge_sound)
                centermess = es.getplayername(event_var['attacker']) + ' got revenge on ' + es.getplayername(event_var['userid']) + '!'              
            del dominations[str2]
        
        if dominations[str1] >= first_range:
            if str1 in domination_records.keys():
                if dominations[str1] > domination_records[str1]:
                    domination_records[str1] = dominations[str1]
            else:
                domination_records[str1] = dominations[str1]
                
        for dom_range in dom_ranges:
            if dominations[str1] == dom_range:
                if dom_range == first_range:
                    if str1 in domination_counts.keys():
                        domination_counts[str1] += 1
                    else:
                        domination_counts[str1] = 1
                if dom_sound_mode > 0:
                    play_sound_local(dom_nemesis_sound, event_var['attacker'], event_var['userid'])
                else:
                    play_sound(dom_nemesis_sound)
                if es.getplayerteam(event_var['userid']) == es.getplayerteam(event_var['attacker']):
                    es.msg('#multi', '#green[DOM]#default ' + es.getplayername(event_var['userid']) + ' is being#lightgreen back-stabbed#default by#green ' + es.getplayername(event_var['attacker']) + '#default!')
                    centermess = es.getplayername(event_var['userid']) + ' is being back-stabbed by ' + es.getplayername(event_var['attacker']) + '!'              
                    es.tell(event_var['userid'], '#multi', '#green[DOM] Your teammate is back-stabbing you!')
                    es.tell(event_var['userid'], '#multi', '#green[DOM] They will be punished for their sins!')
                    es.tell(event_var['attacker'], '#multi', '#green[DOM] You are back-stabbing!')
                    es.tell(event_var['attacker'], '#multi', '#green[DOM] For this you will be punished with death!')
                    gamethread.delayed(1.0, es.sexec, (event_var['attacker'], 'kill'))
                    del dominations[str1]
                else:
                    if dom_sound_mode > 0:
                        gamethread.delayed(dom_taunt_delay, play_taunt_local, (event_var['attacker'], event_var['userid']))
                    else:
                        gamethread.delayed(dom_taunt_delay, play_taunt)
                    es.msg('#multi', '#green[DOM]#default ' + es.getplayername(event_var['userid']) + ' is being#lightgreen ' + dom_ranges[dom_range] + '#default by#green ' + es.getplayername(event_var['attacker']) + '#default!')
                    centermess = es.getplayername(event_var['userid']) + ' is being ' + dom_ranges[dom_range] + ' by ' + es.getplayername(event_var['attacker']) + '!'
                    
                    if dom_range == first_range:              
                        es.tell(event_var['userid'], '#multi', '#green[DOM] You have a#lightgreen new nemesis!')
                        
                    es.tell(event_var['userid'], '#multi', '#green[DOM] Get your revenge on#lightgreen ' + es.getplayername(event_var['attacker']) + '#green now!!')


    else:
        dominations[str1] = 1
        if str2 in dominations.keys():
            if dominations[str2] >= first_range:
                es.msg('#multi', '#green[DOM]#default ' + es.getplayername(event_var['attacker']) + ' got revenge on#green ' + es.getplayername(event_var['userid']) + '#default!')
                if dom_sound_mode > 0:
                    play_sound_local(dom_revenge_sound, event_var['attacker'], event_var['userid'])
                else:
                    play_sound(dom_revenge_sound)
                centermess = es.getplayername(event_var['attacker']) + ' got revenge on ' + es.getplayername(event_var['userid']) + '!'              
            del dominations[str2]
    
    if centermess != '':
        if dom_show_centre_locally:
            gamethread.delayed(0.1, es.centertell, (event_var['attacker'], centermess))
            gamethread.delayed(0.1, es.centertell, (event_var['userid'], centermess))
            gamethread.delayed(1.1, es.centertell, (event_var['attacker'], centermess))
            gamethread.delayed(1.1, es.centertell, (event_var['userid'], centermess))
            gamethread.delayed(2.1, es.centertell, (event_var['attacker'], centermess))
            gamethread.delayed(2.1, es.centertell, (event_var['userid'], centermess))
            gamethread.delayed(3.1, es.centertell, (event_var['attacker'], centermess))
            gamethread.delayed(3.1, es.centertell, (event_var['userid'], centermess))
            gamethread.delayed(4.1, es.centertell, (event_var['attacker'], centermess))
            gamethread.delayed(4.1, es.centertell, (event_var['userid'], centermess))
        else:
            gamethread.delayed(0.1, es.centermsg, centermess)
            gamethread.delayed(1.1, es.centermsg, centermess)
            gamethread.delayed(2.1, es.centermsg, centermess)
            gamethread.delayed(3.1, es.centermsg, centermess)
            gamethread.delayed(4.1, es.centermsg, centermess)

def show_help():
    global dom_popup
    
    user = es.getcmduserid()

    if popuplib.exists('dom_popup'):
        popuplib.delete('dom_popup')

    dom_popup = popuplib.create('dom_popup')
    dom_popup.addline("Dominated! for CS:S v" + info.version)
    dom_popup.addline("Command list")
    dom_popup.addline(" ")
    dom_popup.addline("Say '" + dom_popup_current_cmd + "' to show current dominations.")
    dom_popup.addline("Say '" + dom_popup_most_cmd + "' to show most dominations.")
    dom_popup.addline("Say '" + dom_popup_best_cmd + "' to show best dominations.")
    dom_popup.addline("Say '" + dom_popup_mydoms_cmd + "' to show your dominations.")
    
    if dom_use_sounds:
        dom_popup.addline("Say '" + dom_sound_mute_cmd + "' to mute the domination sounds.")
        dom_popup.addline("Say '" + dom_sound_unmute_cmd + "' to unmute the domination sounds.")

    dom_popup.addline("Say '" + dom_popup_list_ranges_cmd + "' to show the list of domination levels.")
    
    dom_popup.send(user)
    gamethread.delayed(15.0, dom_popup.unsend, user)   

def show_levels():
    global dom_popup
    
    user = es.getcmduserid()
    
    if popuplib.exists('dom_popup'):
        popuplib.delete('dom_popup')
    
    dom_popup = popuplib.create('dom_popup')
    dom_popup.addline("Domination levels")
    dom_popup.addline(" ")
    
    key_list = dom_ranges.keys()
    key_list.sort()
    for key in key_list:
        dom_popup.addline("At " +  str(key) + " kills, you will be " + dom_ranges[key] + " by someone.")
    
    dom_popup.send(user)
    gamethread.delayed(10.0, dom_popup.unsend, user)

def show_current_dominations():
    global dom_popup

    user = es.getcmduserid()
    
    if popuplib.exists('dom_popup'):
        popuplib.delete('dom_popup')

    dom_popup = popuplib.create('dom_popup')
    dom_popup.addline("Current Dominations")
    dom_popup.addline(" ")
    
    key_list = getsortkey(dominations)
    
    count = 0 
    for key in key_list:
        split_key = key.split('<')
        if dominations[key] >= first_range:
            count += 1
            dom_popup.addline(es.getplayername(split_key[1]) + " is dominating " + es.getplayername(split_key[0]) + " (" + str(dominations[key]) + ")")    
    if count == 0:
        dom_popup.addline("No dominations")
    
    dom_popup.send(user)
    gamethread.delayed(5.0, dom_popup.unsend, user)
    
def show_your_dominations():
    global dom_popup
    
    user = es.getcmduserid()
    
    if popuplib.exists('dom_popup'):
        popuplib.delete('dom_popup')
    
    dom_popup = popuplib.create('dom_popup')
    dom_popup.addline("Your Dominations")

    key_list = getsortkey(dominations)
    count = 0 
    for key in key_list:
        split_key = key.split('<')
        if dominations[key] >= first_range and str(user) == split_key[1]:
            if es.getplayername(split_key[0]) != 0:
                count += 1
                dom_popup.addline(es.getplayername(split_key[0]) + " (" + str(dominations[key]) + ")")
                
    if count == 0:
        dom_popup.addline("No dominations")
        
    dom_popup.addline(" ")
    dom_popup.addline("Your Nemesis'")

    count = 0 
    for key in key_list:
        split_key = key.split('<')
        if dominations[key] >= first_range and str(user) == split_key[0]:
            if es.getplayername(split_key[1]) != 0:
                count += 1
                dom_popup.addline(es.getplayername(split_key[1]) + " (" + str(dominations[key]) + ")")
                
    if count == 0:
        dom_popup.addline("No nemesis'")
    
    dom_popup.send(user)
    gamethread.delayed(5.0, dom_popup.unsend, user)
    
def show_most_dominations():
    global dom_popup
    
    user = es.getcmduserid()
    
    if popuplib.exists('dom_popup'):
        popuplib.delete('dom_popup')
    
    dom_popup = popuplib.create('dom_popup')
    dom_popup.addline("Most Dominations")
    dom_popup.addline(" ")
    
    key_list = getsortkey(domination_counts)
    count = 0
    for key in key_list:
        split_key = key.split('<')
        if es.getplayername(split_key[0]) != 0 and es.getplayername(split_key[1]) != 0:
            count += 1
            dom_popup.addline('->' + str(count) + '. ' + es.getplayername(split_key[1]) + ' dominating ' + es.getplayername(split_key[0]) + ' ' + str(domination_counts[key]) + ' times')
            if count == dom_popup_top_value:
                break
    
    if count == 0:
        dom_popup.addline('No dominations')
    
    dom_popup.send(user)
    gamethread.delayed(5.0, dom_popup.unsend, user)

def show_best_dominations():
    global dom_popup
    
    user = es.getcmduserid()
    
    if popuplib.exists('dom_popup'):
        popuplib.delete('dom_popup')
    
    dom_popup = popuplib.create('dom_popup')
    dom_popup.addline("Best Dominations")
    dom_popup.addline(" ")
    
    key_list = getsortkey(domination_records)
    count = 0
    for key in key_list:
        split_key = key.split('<')
        if domination_records[key] >= first_range:
            if es.getplayername(split_key[0]) != 0 and es.getplayername(split_key[1]) != 0:
                count += 1
                dom_popup.addline('->' + str(count) + '. ' + str(domination_records[key]) + ' (' + es.getplayername(split_key[1]) + ' dominating ' + es.getplayername(split_key[0]) + ')')
                if count == dom_popup_top_value:
                    break

    if count == 0:
        dom_popup.addline('No dominations')
    
    dom_popup.send(user)
    gamethread.delayed(5.0, dom_popup.unsend, user)

def getsortkey(d):
    rev_items = [(v,k) for k, v in d.items()]
    rev_items.sort(reverse=True)
    return [k for (v,k) in rev_items]

if dom_use_sounds:
    def mute_sounds():
        global muted
        user = es.getcmduserid()
        es.tell(user, "#multi", "#green[DOM]#default You are now muting dominating sounds.")
        es.tell(user, "#multi", "#green[DOM]#default If you change your mind, you can say " + dom_sound_unmute_cmd + " to unmute to dominating sounds.")
        muted[user] = 1
    
    def unmute_sounds():
        global muted
        user = es.getcmduserid()
        es.tell(user, "#multi", "#green[DOM]#default You are now unmuting dominating sounds.")
        es.tell(user, "#multi", "#green[DOM]#default If you change your mind, you can say " + dom_sound_mute_cmd + " to mute to dominating sounds.")
        if user in muted.keys():
            del muted[user]    
